from httmock import with_httmock
import pysolr
import time

from django.conf import settings
from django.test import LiveServerTestCase as TestCase

from owslib.csw import CatalogueServiceWeb

from hypermap.aggregator.models import Service, Layer
import hypermap.aggregator.tests.mocks.wms
import hypermap.aggregator.tests.mocks.warper
import hypermap.aggregator.tests.mocks.worldmap
from hypermap.aggregator.tasks import index_all_layers


@with_httmock(hypermap.aggregator.tests.mocks.wms.resource_get)
def create_wms_service():
    service = Service(
        type='OGC:WMS',
        url='http://wms.example.com/ows?',
    )
    service.save()


@with_httmock(hypermap.aggregator.tests.mocks.warper.resource_get)
def create_warper_service():
    service = Service(
        type='Hypermap:WARPER',
        url='http://warper.example.com/warper/maps',
    )
    service.save()


@with_httmock(hypermap.aggregator.tests.mocks.worldmap.resource_get)
def create_wm_service():
    service = Service(
        type='Hypermap:WorldMap',
    )
    service.save()


class SolrTest(TestCase):

    """
    Tests Solr integration.
    For now it is needed to manually create a solr core named 'hypermap_test'.
    Later we will programmatically create it and destroy it after testing.
    """

    @with_httmock(hypermap.aggregator.tests.mocks.wms.resource_get)
    def setUp(self):
        solr_url = settings.SEARCH_URL
        self.solr = pysolr.Solr(solr_url, timeout=60)
        create_wms_service()
        create_warper_service()
        create_wm_service()
        # index all
        index_all_layers()
        # some time sleep to commit docs
        time.sleep(10)

    def tearDown(self):
        pass

    def test_solr_sync(self):
        nlayers = Layer.objects.all().count()
        # layers indexed in solr must be same number in django db
        results = self.solr.search(q='*:*')
        self.assertEqual(results.hits, nlayers)
        # layers with invalid bbox don't have the bbox attribute in solr
        nlayers_valid_coordinates = sum(layer.has_valid_bbox() for layer in Layer.objects.all())
        results = self.solr.search(q='bbox:*')
        self.assertEqual(results.hits, nlayers_valid_coordinates)


class CSWTest(TestCase):
    """
    Test CSW endpoint
    """

    def setUp(self):
        """setup records and CSW"""

        self.csw = CatalogueServiceWeb(settings.REGISTRY_PYCSW['server']['url'])

    def tearDown(self):
        """shutdown endpoint and clean out records"""

        Service.objects.all().delete()

    def test_capabilities(self):
        """verify that HHypermap's CSW works properly"""

        # test that OGC:CSW URLs are identical to what is defined in settings
        for op in self.csw.operations:
            for method in op.methods:
                self.assertEqual(settings.REGISTRY_PYCSW['server']['url'], method['url'], 'Expected URL equality')

        # test that OGC:CSW 2.0.2 is supported
        self.assertEqual(self.csw.version, '2.0.2', 'Expected "2.0.2" as a supported version')

        # test that transactions are supported
        transaction = self.csw.get_operation_by_name('Transaction')
        harvest = self.csw.get_operation_by_name('Harvest')

        # test that HHypermap Service types are Harvestable
        for restype in ['http://www.opengis.net/wms', 'http://www.opengis.net/wmts/1.0',
                        'urn:x-esri:serviceType:ArcGIS:MapServer', 'urn:x-esri:serviceType:ArcGIS:ImageServer']:
            self.assertIn(restype, harvest.parameters['ResourceType']['values'])
            self.assertIn(restype, transaction.parameters['TransactionSchemas']['values'])



import sys
import pysolr
import requests
import logging
import json

from django.conf import settings

from hypermap.aggregator.utils import layer2dict

SEARCH_URL = settings.REGISTRY_SEARCH_URL.split('+')[1]


LOGGER = logging.getLogger(__name__)


class SolrHypermap(object):

    def layers_to_solr(self, layers):
        """
        Sync n layers in Solr.
        """
        layers_list = []
        for layer in layers:
            layer_dict = layer2dict(layer)
            layers_list.append(layer_dict)
        layers_json = json.dumps(layers_list)
        try:
            url_solr_update = '%s/solr/hypermap/update/json/docs' % SEARCH_URL
            headers = {"content-type": "application/json"}
            params = {"commitWithin": 1500}
            requests.post(url_solr_update, data=layers_json, params=params, headers=headers)
            LOGGER.info("Solr synced for the given layers.")
            return True, None
        except Exception, e:
            LOGGER.error("Error saving solr records")
            LOGGER.error(e, exc_info=True)
            return False, sys.exc_info()[1]

    def layer_to_solr(self, layer):
        """
        Sync a layer in Solr.
        """
        layer_dict = layer2dict(layer)
        layer_json = json.dumps(layer_dict)
        try:
            url_solr_update = '%s/solr/hypermap/update/json/docs' % SEARCH_URL
            headers = {"content-type": "application/json"}
            params = {"commitWithin": 1500}
            res = requests.post(url_solr_update, data=layer_json, params=params,  headers=headers)
            res = res.json()
            if 'error' in res:
                message = res["error"].get("msg")
                LOGGER.debug("ERROR: {0}".format(message))
                return False, message
            else:
                LOGGER.info("Solr record saved for layer with id: %s" % layer.id)

            return True, None
        except Exception, e:
            LOGGER.error("Error saving solr record for layer with id: %s" % layer.id)
            LOGGER.error(e, exc_info=True)
            return False, sys.exc_info()[1]

    def clear_solr(self, catalog="hypermap"):
        """Clear all indexes in the solr core"""
        solr_url = "{0}/solr/{1}".format(SEARCH_URL, catalog)
        solr = pysolr.Solr(solr_url, timeout=60)
        solr.delete(q='*:*')
        LOGGER.debug('Solr core cleared')

    def update_schema(self, catalog="hypermap"):
        """
        set the mapping in solr.
        :param catalog: core
        :return:
        """
        schema_url = "{0}/solr/{1}/schema".format(SEARCH_URL, catalog)

        fields = [
            {"name": "abstract", "type": "string"},
            {"name": "area", "type": "tdouble"},
            {"name": "availability", "type": "string"},
            {"name": "bbox", "type": "location_rpt"},
            {"name": "domain_name", "type": "string"},
            {"name": "id", "type": "tlong", "required": True},
            {"name": "is_public", "type": "boolean"},
            {"name": "last_status", "type": "boolean"},
            {"name": "layer_date", "type": "tdate", "docValues": True},
            {"name": "layer_datetype", "type": "string"},
            {"name": "layer_id", "type": "tlong"},
            {"name": "layer_originator", "type": "string"},
            {"name": "location", "type": "string"},
            {"name": "max_x", "type": "tdouble"},
            {"name": "max_y", "type": "tdouble"},
            {"name": "min_x", "type": "tdouble"},
            {"name": "min_y", "type": "tdouble"},
            {"name": "name", "type": "string"},
            {"name": "recent_reliability", "type": "tdouble"},
            {"name": "reliability", "type": "tdouble"},
            {"name": "service_id", "type": "tlong"},
            {"name": "service_type", "type": "string"},
            {"name": "srs", "type": "string", "multiValued": True},
            {"name": "tile_url", "type": "string"},
            {"name": "title", "type": "string"},
            {"name": "type", "type": "string"},
            {"name": "url", "type": "string"},
            {"name": "layer_username", "type": "string"},
            {"name": "layer_category", "type": "string"},
            {"name": "centroid_y", "type": "tdouble"},
            {"name": "centroid_x", "type": "tdouble"},
        ]

        headers = {
            "Content-type": "application/json"
        }

        for field in fields:
            data = {
                "add-field": field
            }
            requests.post(schema_url, json=data, headers=headers)



import urllib2
import json
import pika
import logging

from django.conf import settings
from django.http import HttpResponse
from django.template import RequestContext, loader
from django.shortcuts import render
from django.shortcuts import get_object_or_404
from django.db.models import Count
from django.contrib.auth.decorators import login_required
from djmp.views import get_mapproxy


from models import Service, Layer, Catalog
from tasks import (check_all_services, check_service, check_layer, remove_service_checks,
                   index_service, index_all_layers, index_layer, index_cached_layers, clear_index,
                   SEARCH_TYPE, SEARCH_URL)
from enums import SERVICE_TYPES

from hypermap import celeryapp

LOGGER = logging.getLogger(__name__)


def serialize_checks(check_set):
    """
    Serialize a check_set for raphael
    """
    check_set_list = []
    for check in check_set.all()[:25]:
        check_set_list.append(
            {
                'datetime': check.checked_datetime.isoformat(),
                'value': check.response_time,
                'success': 1 if check.success else 0
            }
        )
    return check_set_list


@login_required
def domains(request):
    url = ('%s/select?q=*:*&facet=true&facet.limit=-1&facet.pivot=domain_name,service_id&wt=json&indent=true&rows=0'
           % SEARCH_URL)
    LOGGER.debug(url)
    response = urllib2.urlopen(url)
    data = response.read().replace('\n', '')
    # stats
    layers_count = Layer.objects.all().count()
    services_count = Service.objects.all().count()
    template = loader.get_template('aggregator/index.html')
    context = RequestContext(request, {
        'data': data,
        'layers_count': layers_count,
        'services_count': services_count,
    })
    return HttpResponse(template.render(context))


def index(request, catalog_slug=None):
    order_by = request.GET.get('order_by', '-last_updated')
    filter_by = request.GET.get('filter_by', None)
    query = request.GET.get('q', None)

    services = Service.objects.all()
    if catalog_slug:
        services = Service.objects.filter(catalog__slug=catalog_slug)

    # order_by
    if 'total_checks' in order_by:
        services = services.annotate(total_checks=Count('resource_ptr__check')).order_by(order_by)
    elif 'layers_count' in order_by:
        services = services.annotate(layers_count=Count('layer')).order_by(order_by)
    else:
        services = services.order_by(order_by)
    # filter_by
    if filter_by:
        services = services.filter(type__exact=filter_by)
    # query
    if query:
        services = services.filter(url__icontains=query)
    # types filter
    types_list = []
    for service_type in SERVICE_TYPES:
        type_item = []
        service_type_code = service_type[0]
        type_item.append(service_type_code)
        type_item.append(service_type[1])
        type_item.append(Service.objects.filter(type__exact=service_type_code).count())
        types_list.append(type_item)

    # stats
    layers_count = Layer.objects.all().count()
    services_count = Service.objects.all().count()

    template = loader.get_template('aggregator/search.html')
    context = RequestContext(request, {
        'services': services,
        'types_list': types_list,
        'layers_count': layers_count,
        'services_count': services_count,
        'catalogs': Catalog.objects.filter(url__isnull=False)
    })
    return HttpResponse(template.render(context))


def service_detail(request, catalog_slug, service_id):
    service = get_object_or_404(Service,
                                pk=service_id,
                                catalog__slug=catalog_slug)

    if request.method == 'POST':
        if 'check' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                check_service(service)
            else:
                check_service.delay(service)
        if 'remove' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                remove_service_checks(service)
            else:
                remove_service_checks.delay(service)
        if 'index' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                index_service(service)
            else:
                index_service.delay(service)

    return render(request, 'aggregator/service_detail.html', {'service': service,
                                                              'SEARCH_TYPE': SEARCH_TYPE,
                                                              'SEARCH_URL': SEARCH_URL.rstrip('/'),
                                                              'catalog_slug': catalog_slug})


def service_checks(request, catalog_slug, service_id):
    service = get_object_or_404(Service,
                                pk=service_id,
                                catalog__slug=catalog_slug)
    resource = serialize_checks(service.check_set)

    return render(request, 'aggregator/service_checks.html', {'service': service, 'resource': resource})


def layer_detail(request, catalog_slug, layer_id):
    layer = get_object_or_404(Layer,
                              pk=layer_id,
                              catalog__slug=catalog_slug)

    if request.method == 'POST':
        if 'check' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                check_layer(layer)
            else:
                check_layer.delay(layer)
        if 'remove' in request.POST:
            layer.check_set.all().delete()
        if 'index' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                index_layer(layer)
            else:
                index_layer.delay(layer)

    return render(request, 'aggregator/layer_detail.html', {'layer': layer,
                                                            'SEARCH_TYPE': SEARCH_TYPE,
                                                            'SEARCH_URL': SEARCH_URL.rstrip('/'),
                                                            'catalog_slug': catalog_slug})


def layer_checks(request, catalog_slug, layer_id):
    layer = get_object_or_404(Layer,
                              pk=layer_id,
                              catalog__slug=catalog_slug)
    resource = serialize_checks(layer.check_set)

    return render(request, 'aggregator/layer_checks.html', {'layer': layer, 'resource': resource})


@login_required
def celery_monitor(request):
    """
    A raw celery monitor to figure out which processes are active and reserved.
    """
    inspect = celeryapp.app.control.inspect()
    active_json = inspect.active()
    reserved_json = inspect.reserved()
    active_tasks = []
    if active_json:
        for worker in active_json.keys():
            for task in active_json[worker]:
                id = task['id']
                # not sure why these 2 fields are not already in AsyncResult
                name = task['name']
                time_start = task['time_start']
                args = task['args']
                active_task = celeryapp.app.AsyncResult(id)
                active_task.name = name
                active_task.args = args
                active_task.worker = '%s, pid: %s' % (worker, task['worker_pid'])
                active_task.time_start = time_start
                task_id_sanitized = id.replace('-', '_')
                active_task.task_id_sanitized = task_id_sanitized
                active_tasks.append(active_task)
    reserved_tasks = []
    if reserved_json:
        for worker in active_json.keys():
            for task in reserved_json[worker]:
                id = task['id']
                name = task['name']
                args = task['args']
                reserved_task = celeryapp.app.AsyncResult(id)
                reserved_task.name = name
                reserved_task.args = args
                reserved_tasks.append(reserved_task)

    if request.method == 'POST':
        if 'check_all' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                check_all_services()
            else:
                check_all_services.delay()
        if 'index_all' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                index_all_layers()
            else:
                index_all_layers.delay()
        if 'index_cached' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                index_cached_layers()
            else:
                index_cached_layers.delay()
        if 'clear_index' in request.POST:
            if settings.REGISTRY_SKIP_CELERY:
                clear_index()
            else:
                clear_index.delay()
    return render(
        request,
        'aggregator/celery_monitor.html',
        {
            'active_tasks': active_tasks,
            'reserved_tasks': reserved_tasks,
            'jobs': get_queued_jobs_number(),
        }
    )


def get_queued_jobs_number():
    # to detect tasks in the queued the only way is to use amqplib so far
    from amqplib import client_0_8 as amqp

    params = pika.URLParameters(settings.BROKER_URL)

    conn = amqp.Connection(host='{0}:{1}'.format(params.host, params.port),
                           userid=params.credentials.username,
                           password=params.credentials.password,
                           virtual_host=params.virtual_host,
                           insist=False)
    chan = conn.channel()
    name, jobs, consumers = chan.queue_declare(queue="hypermap", passive=False)
    return jobs


@login_required
def update_jobs_number(request):
    response_data = {}
    response_data['jobs'] = get_queued_jobs_number()
    json_data = json.dumps(response_data)
    return HttpResponse(json_data, content_type="application/json")


@login_required
def update_progressbar(request, task_id):
    response_data = {}
    active_task = celeryapp.app.AsyncResult(task_id)
    progressbar = 100
    status = '100%'
    state = 'COMPLETED'
    if active_task:
        if not active_task.ready():
            current = active_task.info['current']
            total = active_task.info['total']
            progressbar = (current / float(total) * 100)
            status = "%s/%s (%.2f %%)" % (current, total, progressbar)
            state = active_task.state
    response_data['progressbar'] = progressbar
    response_data['status'] = status
    response_data['state'] = state
    json_data = json.dumps(response_data)
    return HttpResponse(json_data, content_type="application/json")


def layer_mapproxy(request, catalog_slug, layer_id, path_info):
    # Get Layer with matching catalog and primary key
    layer = get_object_or_404(Layer,
                              pk=layer_id,
                              catalog__slug=catalog_slug)

    # Set up a mapproxy app for this particular layer
    mp, yaml_config = get_mapproxy(layer)

    query = request.META['QUERY_STRING']

    if len(query) > 0:
        path_info = path_info + '?' + query

    params = {}
    headers = {
            'X-Script-Name': '/registry/{0}/layer/{1}/map/'.format(catalog_slug, layer.id),
            'X-Forwarded-Host': request.META['HTTP_HOST'],
            'HTTP_HOST': request.META['HTTP_HOST'],
            'SERVER_NAME': request.META['SERVER_NAME'],
            }

    if path_info == '/config':
        response = HttpResponse(yaml_config, content_type='text/plain')
        return response

    # Get a response from MapProxy as if it was running standalone.
    mp_response = mp.get(path_info, params, headers)

    # Create a Django response from the MapProxy WSGI response.
    response = HttpResponse(mp_response.body, status=mp_response.status_int)
    for header, value in mp_response.headers.iteritems():
        response[header] = value

    return response



# -*- coding: utf-8 -*-

"""
Tests for metadata constructs across models
"""

import unittest

from httmock import with_httmock
from lxml import etree

from owslib.csw import CswRecord

from hypermap.aggregator.models import gen_anytext, Layer, Service, Catalog
import hypermap.aggregator.tests.mocks.wms
import hypermap.aggregator.tests.mocks.warper
import hypermap.aggregator.tests.mocks.worldmap


@with_httmock(hypermap.aggregator.tests.mocks.wms.resource_get)
def create_wms_service():

    catalog, created = Catalog.objects.get_or_create(
        name="hypermap", slug="hypermap",
        url="search_api"
    )

    service = Service(
        type='OGC:WMS',
        url='http://wms.example.com/ows?',
        catalog=Catalog.objects.get()
    )
    service.save()


@with_httmock(hypermap.aggregator.tests.mocks.warper.resource_get)
def create_warper_service():

    catalog, created = Catalog.objects.get_or_create(
        name="hypermap", slug="hypermap",
        url="search_api"
    )

    service = Service(
        type='Hypermap:WARPER',
        url='http://warper.example.com/warper/maps',
        catalog=catalog
    )
    service.save()


@with_httmock(hypermap.aggregator.tests.mocks.worldmap.resource_get)
def create_wm_service():
    catalog, created = Catalog.objects.get_or_create(
        name="hypermap", slug="hypermap",
        url="search_api"
    )
    service = Service(
        type='Hypermap:WorldMap',
        catalog=catalog
    )
    service.save()


class TestMetadata(unittest.TestCase):
    def setUp(self):
        """setup test data"""
        Service.objects.all().delete()
        create_warper_service()
        create_wm_service()
        create_wms_service()

    def tearDown(self):
        """delete test data"""
        Service.objects.all().delete()

    def test_service_fields(self):
        """test Service metadata fields"""
        layer = Service.objects.filter(type='OGC:WMS').all()[0]
        self.assertIsNotNone(layer.xml, 'Expected XML document')

        xml = etree.fromstring(layer.xml)
        csw_record = CswRecord(xml)

        self.assertIsInstance(xml, etree._Element, 'Expected lxml instance')
        self.assertEqual(layer.title, csw_record.title, 'Expected title equality')
        self.assertEqual(layer.abstract, csw_record.abstract, 'Expected abstract equality')
        self.assertEqual(layer.csw_type, 'service', 'Expected CSW type equality')
        self.assertEqual(layer.csw_typename, 'csw:Record', 'Expected CSW typename equality')
        self.assertEqual(layer.csw_schema, 'http://www.opengis.net/cat/csw/2.0.2', 'Expected CSW schema equality')

        anytext = gen_anytext(layer.title, layer.abstract)
        self.assertEqual(anytext, layer.anytext, 'Expected anytext equality')

    def test_layer_fields(self):
        """test Layer metadata fields"""
        layer = Layer.objects.filter(type='OGC:WMS').all()[0]
        self.assertIsNotNone(layer.xml, 'Expected XML document')

        xml = etree.fromstring(layer.xml)
        csw_record = CswRecord(xml)

        self.assertIsInstance(xml, etree._Element, 'Expected lxml instance')
        self.assertEqual(layer.title, csw_record.title, 'Expected title equality')
        self.assertEqual(layer.abstract, csw_record.abstract, 'Expected abstract equality')
        self.assertEqual(layer.csw_type, 'dataset', 'Expected CSW type equality')
        self.assertEqual(layer.csw_typename, 'csw:Record', 'Expected CSW typename equality')
        self.assertEqual(layer.csw_schema, 'http://www.opengis.net/cat/csw/2.0.2', 'Expected CSW schema equality')

        self.assertEqual(layer.service.id, int(csw_record.relation), 'Expected relation equality')
        self.assertEqual(layer.url, csw_record.source, 'Expected URL/source equality')

        anytext = gen_anytext(layer.title, layer.abstract, list(layer.keywords.names()).sort())
        self.assertEqual(anytext, layer.anytext, 'Expected anytext equality: %s != %s' % (anytext, layer.anytext))

if __name__ == '__main__':
    unittest.main()



import re

import datetime
import isodate
import math

import requests
from dateutil.parser import parse
from shapely.geometry import box


def is_range_common_era(start, end):
    """
    does the range contains CE dates.
    BCE and CE are not compatible at the moment.
    :param start:
    :param end:
    :return: False if contains BCE dates.
    """
    return all([start.get("is_common_era"),
                end.get("is_common_era")])


def parse_datetime(date_str):
    """
    Parses a date string to date object.
    for BCE dates, only supports the year part.
    """
    is_common_era = True
    date_str_parts = date_str.split("-")
    if date_str_parts and date_str_parts[0] == '':
        is_common_era = False
        # for now, only support BCE years

        # assume the datetime comes complete, but
        # when it comes only the year, add the missing datetime info:
        if len(date_str_parts) == 2:
            date_str = date_str + "-01-01T00:00:00Z"

    parsed_datetime = {
        'is_common_era': is_common_era,
        'parsed_datetime': None
    }

    if is_common_era:
        if date_str == '*':
            return parsed_datetime  # open ended.

        default = datetime.datetime.now().replace(
            hour=0, minute=0, second=0, microsecond=0,
            day=1, month=1
        )
        parsed_datetime['parsed_datetime'] = parse(date_str, default=default)
        return parsed_datetime

    parsed_datetime['parsed_datetime'] = date_str
    return parsed_datetime


def parse_solr_time_range_as_pair(time_filter):
    """
    :param time_filter: [2013-03-01 TO 2013-05-01T00:00:00]
    :return: (2013-03-01, 2013-05-01T00:00:00)
    """
    pattern = "\\[(.*) TO (.*)\\]"
    matcher = re.search(pattern, time_filter)
    if matcher:
        return matcher.group(1), matcher.group(2)
    else:
        raise Exception("Regex {0} couldn't parse {1}".format(pattern, time_filter))


def parse_datetime_range(time_filter):
    """
    Parse the url param to python objects.
    From what time range to divide by a.time.gap into intervals.
    Defaults to q.time and otherwise 90 days.
    Validate in API: re.search("\\[(.*) TO (.*)\\]", value)
    :param time_filter: [2013-03-01 TO 2013-05-01T00:00:00]
    :return: datetime.datetime(2013, 3, 1, 0, 0), datetime.datetime(2013, 5, 1, 0, 0)
    """

    if not time_filter:
        time_filter = "[* TO *]"

    start, end = parse_solr_time_range_as_pair(time_filter)
    start, end = parse_datetime(start), parse_datetime(end)
    return start, end


def parse_datetime_range_to_solr(time_filter):
    start, end = parse_datetime_range(time_filter)
    left = "*"
    right = "*"

    if start.get("parsed_datetime"):
        left = start.get("parsed_datetime")
        if start.get("is_common_era"):
            left = start.get("parsed_datetime").isoformat().replace("+00:00", "") + 'Z'

    if end.get("parsed_datetime"):
        right = end.get("parsed_datetime")
        if end.get("is_common_era"):
            right = end.get("parsed_datetime").isoformat().replace("+00:00", "") + 'Z'

    return "[{0} TO {1}]".format(left, right)


def parse_ISO8601(time_gap):
    """
    P1D to (1, ("DAYS", isodate.Duration(days=1)).
    P1Y to (1, ("YEARS", isodate.Duration(years=1)).
    :param time_gap: ISO8601 string.
    :return: tuple with quantity and unit of time.
    """
    matcher = None

    if time_gap.count("T"):
        units = {
            "H": ("HOURS", isodate.Duration(hours=1)),
            "M": ("MINUTES", isodate.Duration(minutes=1)),
            "S": ("SECONDS", isodate.Duration(seconds=1))
        }
        matcher = re.search("PT(\d+)([HMS])", time_gap)
        if matcher:
            quantity = int(matcher.group(1))
            unit = matcher.group(2)
            return quantity, units.get(unit)
        else:
            raise Exception("Does not match the pattern: {}".format(time_gap))
    else:
        units = {
            "Y": ("YEARS", isodate.Duration(years=1)),
            "M": ("MONTHS", isodate.Duration(months=1)),
            "W": ("WEEKS", isodate.Duration(weeks=1)),
            "D": ("DAYS", isodate.Duration(days=1))
        }
        matcher = re.search("P(\d+)([YMWD])", time_gap)
        if matcher:
            quantity = int(matcher.group(1))
            unit = matcher.group(2)
        else:
            raise Exception("Does not match the pattern: {}".format(time_gap))

    return quantity, units.get(unit)


def compute_gap(start, end, time_limit):
    """
    Compute a gap that seems reasonable, considering natural time units and limit.
    # TODO: make it to be reasonable.
    # TODO: make it to be small unit of time sensitive.
    :param start: datetime
    :param end: datetime
    :param time_limit: gaps count
    :return: solr's format duration.
    """
    if is_range_common_era(start, end):
        duration = end.get("parsed_datetime") - start.get("parsed_datetime")
        unit = int(math.ceil(duration.days / float(time_limit)))
        return "+{0}DAYS".format(unit)
    else:
        # at the moment can not do maths with BCE dates.
        # those dates are relatively big, so 100 years are reasonable in those cases.
        # TODO: calculate duration on those cases.
        return "+100YEARS"


def gap_to_elastic(time_gap):
    # elastic units link: https://www.elastic.co/guide/en/elasticsearch/reference/current/common-options.html#time-units
    elastic_units = {
        "YEARS": 'y',
        "MONTHS": 'M',
        "WEEKS": 'w',
        "DAYS": 'd',
        "HOURS": 'h',
        "MINUTES": 'm',
        "SECONDS": 's'
    }
    quantity, unit = parse_ISO8601(time_gap)
    interval = "{0}{1}".format(str(quantity), elastic_units[unit[0]])
    return interval


def gap_to_sorl(time_gap):
    """
    P1D to +1DAY
    :param time_gap:
    :return: solr's format duration.
    """
    quantity, unit = parse_ISO8601(time_gap)
    if unit[0] == "WEEKS":
        return "+{0}DAYS".format(quantity * 7)
    else:
        return "+{0}{1}".format(quantity, unit[0])


def request_time_facet(field, time_filter, time_gap, time_limit=100):
    """
    time facet query builder
    :param field: map the query to this field.
    :param time_limit: Non-0 triggers time/date range faceting. This value is the maximum number of time ranges to
    return when a.time.gap is unspecified. This is a soft maximum; less will usually be returned.
    A suggested value is 100.
    Note that a.time.gap effectively ignores this value.
    See Solr docs for more details on the query/response format.
    :param time_filter: From what time range to divide by a.time.gap into intervals.
    Defaults to q.time and otherwise 90 days.
    :param time_gap: The consecutive time interval/gap for each time range. Ignores a.time.limit.
    The format is based on a subset of the ISO-8601 duration format
    :return: facet.range=manufacturedate_dt&f.manufacturedate_dt.facet.range.start=2006-02-11T15:26:37Z&f.
    manufacturedate_dt.facet.range.end=2006-02-14T15:26:37Z&f.manufacturedate_dt.facet.range.gap=+1DAY
    """
    start, end = parse_datetime_range(time_filter)

    key_range_start = "f.{0}.facet.range.start".format(field)
    key_range_end = "f.{0}.facet.range.end".format(field)
    key_range_gap = "f.{0}.facet.range.gap".format(field)
    key_range_mincount = "f.{0}.facet.mincount".format(field)

    if time_gap:
        gap = gap_to_sorl(time_gap)
    else:
        gap = compute_gap(start, end, time_limit)

    value_range_start = start.get("parsed_datetime")
    if start.get("is_common_era"):
        value_range_start = start.get("parsed_datetime").isoformat().replace("+00:00", "") + "Z"

    value_range_end = start.get("parsed_datetime")
    if end.get("is_common_era"):
        value_range_end = end.get("parsed_datetime").isoformat().replace("+00:00", "") + "Z"

    value_range_gap = gap

    params = {
        'facet.range': field,
        key_range_start: value_range_start,
        key_range_end: value_range_end,
        key_range_gap: value_range_gap,
        key_range_mincount: 1
    }

    return params


def parse_solr_geo_range_as_pair(geo_box_str):
    """
    :param geo_box_str: [-90,-180 TO 90,180]
    :return: ("-90,-180", "90,180")
    """
    pattern = "\\[(.*) TO (.*)\\]"
    matcher = re.search(pattern, geo_box_str)
    if matcher:
        return matcher.group(1), matcher.group(2)
    else:
        raise Exception("Regex {0} could not parse {1}".format(pattern, geo_box_str))


def parse_lat_lon(point_str):
    lat, lon = map(float, point_str.split(','))
    return lat, lon


def parse_geo_box(geo_box_str):
    """
    parses [-90,-180 TO 90,180] to a shapely.geometry.box
    :param geo_box_str:
    :return:
    """

    from_point_str, to_point_str = parse_solr_geo_range_as_pair(geo_box_str)
    from_point = parse_lat_lon(from_point_str)
    to_point = parse_lat_lon(to_point_str)
    rectangle = box(from_point[0], from_point[1], to_point[0], to_point[1])
    return rectangle


def request_heatmap_facet(field, hm_filter, hm_grid_level, hm_limit):
    """
    heatmap facet query builder
    :param field: map the query to this field.
    :param hm_filter: From what region to plot the heatmap. Defaults to q.geo or otherwise the world.
    :param hm_grid_level: To explicitly specify the grid level, e.g. to let a user ask for greater or courser
    resolution than the most recent request. Ignores a.hm.limit.
    :param hm_limit: Non-0 triggers heatmap/grid faceting. This number is a soft maximum on thenumber of
    cells it should have. There may be as few as 1/4th this number in return. Note that a.hm.gridLevel can effectively
    ignore this value. The response heatmap contains a counts grid that can be null or contain null rows when all its
    values would be 0. See Solr docs for more details on the response format.
    :return:
    """

    if not hm_filter:
        hm_filter = '[-90,-180 TO 90,180]'

    params = {
        'facet': 'on',
        'facet.heatmap': field,
        'facet.heatmap.geom': hm_filter
    }

    if hm_grid_level:
        # note: aHmLimit is ignored in this case
        params['facet.heatmap.gridLevel'] = hm_grid_level
    else:
        # Calculate distErr that will approximate aHmLimit many cells as an upper bound
        rectangle = parse_geo_box(hm_filter)
        degrees_side_length = rectangle.length / 2
        cell_side_length = math.sqrt(float(hm_limit))
        cell_side_length_degrees = degrees_side_length / cell_side_length * 2
        params['facet.heatmap.distErr'] = str(float(cell_side_length_degrees))
        # TODO: not sure about if returning correct param values.

    # get_params = urllib.urlencode(params)
    return params


def request_field_facet(field, limit, ex_filter=True):
    pass


def asterisk_to_min_max(field, time_filter, search_engine_endpoint, actual_params=None):
    """
    traduce [* TO *] to something like [MIN-INDEXED-DATE TO MAX-INDEXED-DATE]
    :param field: map the stats to this field.
    :param time_filter: this is the value to be translated. think in "[* TO 2000]"
    :param search_engine_endpoint: solr core
    :param actual_params: (not implemented) to merge with other params.
    :return: translated time filter
    """

    if actual_params:
        raise NotImplemented("actual_params")

    start, end = parse_solr_time_range_as_pair(time_filter)
    if start == '*' or end == '*':
        params_stats = {
            "q": "*:*",
            "rows": 0,
            "stats.field": field,
            "stats": "true",
            "wt": "json"
        }
        res_stats = requests.get(search_engine_endpoint, params=params_stats)

        if res_stats.ok:

            stats_date_field = res_stats.json()["stats"]["stats_fields"][field]
            date_min = stats_date_field["min"]
            date_max = stats_date_field["max"]

            if start != '*':
                date_min = start
            if end != '*':
                date_max = end

            time_filter = "[{0} TO {1}]".format(date_min, date_max)

    return time_filter



import urllib2
import logging
import requests
import re
import sys
import math
import traceback
import datetime
from urlparse import urlparse

from django.utils.html import strip_tags
from django.conf import settings

from lxml import etree
from owslib.csw import CatalogueServiceWeb, CswRecord
from owslib.wms import WebMapService
from owslib.tms import TileMapService
from owslib.wmts import WebMapTileService
from arcrest import Folder as ArcFolder

from hypermap.aggregator.enums import SERVICE_TYPES
from lxml.etree import XMLSyntaxError
from shapely.geometry import box

LOGGER = logging.getLogger(__name__)


def create_layer_from_metadata_xml(resourcetype, xml, monitor=False):
    """
    Create a layer / keyword list from a metadata record if it does not already exist.
    """
    from models import gen_anytext, Layer

    if resourcetype == 'http://www.opengis.net/cat/csw/2.0.2':  # Dublin core
        md = CswRecord(etree.fromstring(xml))

    layer = Layer(
        is_monitored=monitor,
        name=md.title,
        title=md.title,
        abstract=md.abstract,
        xml=xml,
        anytext=gen_anytext(md.title, md.abstract, md.subjects)
    )

    if md.bbox is not None:
        layer.bbox_x0 = format_float(md.bbox.minx)
        layer.bbox_y0 = format_float(md.bbox.miny)
        layer.bbox_x1 = format_float(md.bbox.maxx)
        layer.bbox_y1 = format_float(md.bbox.maxy)

        layer.wkt_geometry = bbox2wktpolygon([md.bbox.minx, md.bbox.miny, md.bbox.maxx, md.bbox.maxy])

    return layer, md.subjects


def create_service_from_endpoint(endpoint, service_type, title=None, abstract=None, catalog=None):
    """
    Create a service from an endpoint if it does not already exists.
    """
    from models import Service
    if Service.objects.filter(url=endpoint, catalog=catalog).count() == 0:
        # check if endpoint is valid
        request = requests.get(endpoint)
        if request.status_code == 200:
            LOGGER.debug('Creating a %s service for endpoint=%s catalog=%s' % (service_type, endpoint, catalog))
            service = Service(
                        type=service_type, url=endpoint, title=title, abstract=abstract,
                        csw_type='service', catalog=catalog
                        )
            service.save()
            return service
        else:
            LOGGER.warning('This endpoint is invalid, status code is %s' % request.status_code)
    else:
        LOGGER.warning('A service for this endpoint %s in catalog %s already exists' % (endpoint, catalog))
        return None


def create_services_from_endpoint(url, catalog, greedy_opt=True):
    """
    Generate service/services from an endpoint.
    WMS, WMTS, TMS endpoints correspond to a single service.
    ESRI, CSW endpoints corrispond to many services.
    :return: imported, message
    """

    # this variable will collect any exception message during the routine.
    # will be used in the last step to send a message if "detected" var is False.
    messages = []

    num_created = 0
    endpoint = get_sanitized_endpoint(url)
    try:
        urllib2.urlopen(endpoint, timeout=10)
    except Exception as e:
        message = traceback.format_exception(*sys.exc_info())
        LOGGER.error('Cannot open this endpoint: %s' % endpoint)
        LOGGER.error('ERROR MESSAGE: %s' % message)
        LOGGER.error(e, exc_info=True)

        return False, message

    detected = False

    # handle specific service types for some domains (WorldMap, Wrapper...)
    parsed_uri = urlparse(endpoint)
    domain = '{uri.scheme}://{uri.netloc}/'.format(uri=parsed_uri)
    if domain == 'http://worldmap.harvard.edu/':
        service_type = 'Hypermap:WorldMap'
        title = 'Harvard WorldMap'
        abstract = 'Harvard WorldMap'
        endpoint = domain
        detected = True
    if domain in [
        'http://maps.nypl.org/',
        'http://mapwarper.net/',
        'http://warp.worldmap.harvard.edu/',
    ]:
        service_type = 'Hypermap:WARPER'
        title = 'Warper at %s' % domain
        abstract = 'Warper at %s' % domain
        detected = True

    # test if it is CSW, WMS, TMS, WMTS or Esri
    # CSW
    try:
        csw = CatalogueServiceWeb(endpoint)
        service_type = 'OGC:CSW'
        service_links = {}
        detected = True

        typenames = 'csw:Record'
        outputschema = 'http://www.opengis.net/cat/csw/2.0.2'

        if 'csw_harvest_pagesize' in settings.REGISTRY_PYCSW['manager']:
            pagesize = int(settings.REGISTRY_PYCSW['manager']['csw_harvest_pagesize'])
        else:
            pagesize = 10

        LOGGER.debug('Harvesting CSW %s' % endpoint)
        # now get all records
        # get total number of records to loop against
        try:
            csw.getrecords2(typenames=typenames, resulttype='hits',
                            outputschema=outputschema)
            matches = csw.results['matches']
        except:  # this is a CSW, but server rejects query
            raise RuntimeError(csw.response)

        if pagesize > matches:
            pagesize = matches

        LOGGER.info('Harvesting %d CSW records' % matches)

        # loop over all catalogue records incrementally
        for r in range(1, matches+1, pagesize):
            LOGGER.info('Parsing %s from %s' % (r, matches))
            try:
                csw.getrecords2(typenames=typenames, startposition=r,
                                maxrecords=pagesize, outputschema=outputschema, esn='full')
            except Exception as err:  # this is a CSW, but server rejects query
                raise RuntimeError(csw.response)
            for k, v in csw.records.items():
                # try to parse metadata
                try:
                    LOGGER.info('Looking for service links')

                    LOGGER.debug('Looking for service links via dct:references')
                    if v.references:
                        for ref in v.references:
                            scheme = None

                            if ref['scheme'] in [st[0] for st in SERVICE_TYPES]:
                                if ref['url'] not in service_links:
                                    scheme = ref['scheme']
                                    service_links[ref['url']] = scheme
                            else:  # loose detection
                                scheme = detect_metadata_url_scheme(ref['url'])
                                if scheme is not None:
                                    if ref['url'] not in service_links:
                                        service_links[ref['url']] = scheme

                            if scheme is None:
                                continue
                            try:
                                service = create_service_from_endpoint(ref['url'], scheme, catalog=catalog)

                                if service is not None:
                                    num_created = num_created + 1
                                    LOGGER.info('Found %s services on endpoint' % num_created)
                            except Exception, e:
                                LOGGER.error('Could not create service for %s : %s' % (scheme, ref['url']))
                                LOGGER.error(e, exc_info=True)
                    LOGGER.debug('Looking for service links via the GeoNetwork-ish dc:URI')
                    if v.uris:
                        for u in v.uris:  # loose detection
                            scheme = detect_metadata_url_scheme(u['url'])
                            if scheme is not None:
                                if u['url'] not in service_links:
                                    service_links[u['url']] = scheme
                            else:
                                continue

                            try:
                                service = create_service_from_endpoint(u['url'], scheme, catalog=catalog)

                                if service is not None:
                                    num_created = num_created + 1
                                    LOGGER.info('Found %s services on endpoint' % num_created)
                            except Exception, e:
                                LOGGER.error('Could not create service for %s : %s' % (scheme, u['url']))
                                LOGGER.error(e, exc_info=True)

                except Exception as err:  # parsing failed for some reason
                    LOGGER.warning('Metadata parsing failed %s', err)
                    LOGGER.error(err, exc_info=True)

    except XMLSyntaxError as e:
        # This is not XML, so likely not a CSW. Moving on.
        pass
    except Exception as e:
        LOGGER.error(e, exc_info=True)
        messages.append(str(e))

    # WMS
    if not detected:
        try:
            service = WebMapService(endpoint, timeout=10)
            service_type = 'OGC:WMS'
            title = service.identification.title,
            abstract = service.identification.abstract
            detected = True
        except XMLSyntaxError as e:
            # This is not XML, so likely not a WMS. Moving on.
            pass
        except Exception as e:
            LOGGER.error(e, exc_info=True)
            messages.append(str(e))

    # TMS
    if not detected:
        try:
            service = TileMapService(endpoint, timeout=10)
            service_type = 'OSGeo:TMS'
            title = service.identification.title,
            abstract = service.identification.abstract
            detected = True
        except XMLSyntaxError as e:
            # This is not XML, so likely not a TsMS. Moving on.
            pass
        except Exception as e:
            LOGGER.error(e, exc_info=True)
            messages.append(str(e))

    # WMTS
    if not detected:
        try:
            # @tomkralidis timeout is not implemented for WebMapTileService?
            service = WebMapTileService(endpoint)
            service_type = 'OGC:WMTS'
            title = service.identification.title,
            abstract = service.identification.abstract
            detected = True
        except XMLSyntaxError as e:
            # This is not XML, so likely not a WMTS. Moving on.
            pass
        except Exception as e:
            LOGGER.error(e, exc_info=True)
            messages.append(str(e))

    # if detected, let's create the service
    if detected and service_type != 'OGC:CSW':
        try:
            service = create_service_from_endpoint(
                endpoint,
                service_type,
                title,
                abstract=abstract,
                catalog=catalog
            )
            if service is not None:
                num_created = num_created + 1
        except XMLSyntaxError as e:
            # This is not XML, so likely not a OGC:CSW. Moving on.
            pass
        except Exception as e:
            LOGGER.error(e, exc_info=True)
            messages.append(str(e))

    # Esri
    # a good sample is here: https://gis.ngdc.noaa.gov/arcgis/rest/services

    # we can safely assume the following condition (at least it is true for 1170 services)
    # we need to test this as ArcFolder can freeze with not esri url such as this one:
    # http://hh.worldmap.harvard.edu/admin/aggregator/service/?q=%2Frest%2Fservices

    if '/rest/services' in endpoint:
        if not detected:
            try:
                esri = ArcFolder(endpoint)
                service_type = 'ESRI'
                detected = True

                service_to_process, folder_to_process = esri.services, esri.folders
                if not greedy_opt:
                    folder_to_process = []
                    service_to_process = get_single_service(esri, url, endpoint)

                processed_services = process_esri_services(service_to_process, catalog)
                num_created = num_created + len(processed_services)

                for folder in folder_to_process:
                    folder_services = process_esri_services(folder.services, catalog)
                    num_created = num_created + len(folder_services)

            except Exception as e:
                LOGGER.error(e, exc_info=True)
                messages.append(str(e))

    if detected:
        return True, '%s service/s created' % num_created
    else:
        m = '|'.join(messages)
        return False, 'ERROR! Could not detect service type for ' \
                      'endpoint %s or already existing. messages=(%s)' % (endpoint, m)


def get_single_service(esri, url, endpoint):
    url_split_list = url.split(endpoint + '/')[1].split('/')
    len_list = len(url_split_list)

    service_name = url_split_list[0]
    if len_list == 3:
        service_to_process = [s for s in esri.services if service_name in s.url]
    elif len_list == 4:
        folder_name, folder_service_name = url_split_list[0], url_split_list[1]
        service_to_process = []
        esri_folder = [f for f in esri.folders if folder_name in f.url]
        if len(esri_folder) != 0:
            for folder in esri_folder:
                service_to_process = [s for s in folder.services if folder_service_name in s.url]

    return service_to_process


def process_esri_services(esri_services, catalog):
    services_created = []
    for esri_service in esri_services:
        # for now we process only MapServer
        if '/MapServer/' in esri_service.url:
            # we import only MapServer with at least one layer
            if hasattr(esri_service, 'layers'):
                service = create_service_from_endpoint(
                    esri_service.url,
                    'ESRI:ArcGIS:MapServer',
                    esri_service.mapName,
                    esri_service.description,
                    catalog=catalog
                )
                services_created.append(service)

        # Don't process ImageServer until the following issue has been resolved:
        # https://github.com/mapproxy/mapproxy/issues/235
        # if '/ImageServer/' in esri_service.url:
        #     service = create_service_from_endpoint(
        #         esri_service.url,
        #         'ESRI:ArcGIS:ImageServer',
        #         '',
        #         esri_service.serviceDescription
        #     )
        #      services_created.append(service)

    return services_created


def inverse_mercator(xy):
    """
        Given coordinates in spherical mercator, return a lon,lat tuple.
    """
    lon = (xy[0] / 20037508.34) * 180
    lat = (xy[1] / 20037508.34) * 180
    lat = 180 / math.pi * \
        (2 * math.atan(math.exp(lat * math.pi / 180)) - math.pi / 2)
    return (lon, lat)


def mercator_to_llbbox(bbox):
    minlonlat = inverse_mercator([bbox[0], bbox[1]])
    maxlonlat = inverse_mercator([bbox[2], bbox[3]])
    return [minlonlat[0], minlonlat[1], maxlonlat[0], maxlonlat[1]]


def get_sanitized_endpoint(url):
    """
    Sanitize an endpoint, as removing unneeded parameters
    """
    # sanitize esri
    sanitized_url = url.rstrip()
    esri_string = '/rest/services'
    if esri_string in url:
        match = re.search(esri_string, sanitized_url)
        sanitized_url = url[0:(match.start(0)+len(esri_string))]
    return sanitized_url


def get_esri_service_name(url):
    """
    A method to get a service name from an esri endpoint.
    For example: http://example.com/arcgis/rest/services/myservice/mylayer/MapServer/?f=json
    Will return: myservice/mylayer
    """
    result = re.search('rest/services/(.*)/[MapServer|ImageServer]', url)
    if result is None:
        return url
    else:
        return result.group(1)


def get_esri_extent(esriobj):
    """
    Get the extent of an ESRI resource
    """

    extent = None
    srs = None

    if 'fullExtent' in esriobj._json_struct:
        extent = esriobj._json_struct['fullExtent']
    if 'extent' in esriobj._json_struct:
        extent = esriobj._json_struct['extent']

    try:
        srs = extent['spatialReference']['wkid']
    except KeyError, err:
        LOGGER.error(err, exc_info=True)

    return [extent, srs]


def flip_coordinates(c1, c2):
    if c1 > c2:
        LOGGER.debug('Flipping coordinates %s, %s' % (c1, c2))
        temp = c1
        c1 = c2
        c2 = temp
    return c1, c2


def format_float(value):
    if value is None:
        return None
    try:
        value = float(value)
        if value > 999999999:
            return None
        return value
    except ValueError:
        return None


def bbox2wktpolygon(bbox):
    """
    Return OGC WKT Polygon of a simple bbox list of strings
    """

    minx = float(bbox[0])
    miny = float(bbox[1])
    maxx = float(bbox[2])
    maxy = float(bbox[3])
    return 'POLYGON((%.2f %.2f, %.2f %.2f, %.2f %.2f, %.2f %.2f, %.2f %.2f))' \
        % (minx, miny, minx, maxy, maxx, maxy, maxx, miny, minx, miny)


def get_solr_date(pydate, is_negative):
    """
    Returns a date in a valid Solr format from a string.
    """
    # check if date is valid and then set it to solr format YYYY-MM-DDThh:mm:ssZ
    try:
        if isinstance(pydate, datetime.datetime):
            solr_date = '%sZ' % pydate.isoformat()[0:19]
            if is_negative:
                LOGGER.debug('%s This layer has a negative date' % solr_date)
                solr_date = '-%s' % solr_date
            return solr_date
        else:
            return None
    except Exception, e:
        LOGGER.error(e, exc_info=True)
        return None


def get_date(layer):
    """
    Returns a custom date representation. A date can be detected or from metadata.
    It can be a range or a simple date in isoformat.
    """
    date = None
    sign = '+'
    date_type = 1
    layer_dates = layer.get_layer_dates()
    # we index the first date!
    if layer_dates:
        sign = layer_dates[0][0]
        date = layer_dates[0][1]
        date_type = layer_dates[0][2]
    if date is None:
        date = layer.created
    # layer date > 2300 is invalid for sure
    # TODO put this logic in date miner
    if date.year > 2300:
        date = None
    if date_type == 0:
        date_type = "Detected"
    if date_type == 1:
        date_type = "From Metadata"
    return get_solr_date(date, (sign == '-')), date_type


def get_domain(url):
    urlParts = urlparse(url)
    hostname = urlParts.hostname
    if hostname == "localhost":
        return "Harvard"  # assumption
    return hostname


def layer2dict(layer):
    """
    Return a json representation for a layer.
    """
    logger = logging.getLogger("hypermap")
    category = None
    username = None
    bbox = None
    if not layer.has_valid_bbox():
        message = 'There are not valid coordinates for layer id: %s' % layer.id
        logger.error(message)
    else:
        bbox = [float(layer.bbox_x0), float(layer.bbox_y0), float(layer.bbox_x1), float(layer.bbox_y1)]
        for proj in layer.service.srs.values():
            if proj['code'] in ('102113', '102100'):
                bbox = mercator_to_llbbox(bbox)
        minX = bbox[0]
        minY = bbox[1]
        maxX = bbox[2]
        maxY = bbox[3]
        # coords hack needed by solr
        if (minX < -180):
            minX = -180
        if (maxX > 180):
            maxX = 180
        if (minY < -90):
            minY = -90
        if (maxY > 90):
            maxY = 90
        rectangle = box(minX, minY, maxX, maxY)
        wkt = "ENVELOPE({:f},{:f},{:f},{:f})".format(minX, maxX, maxY, minY)
        halfWidth = (maxX - minX) / 2.0
        halfHeight = (maxY - minY) / 2.0
        area = (halfWidth * 2) * (halfHeight * 2)
    domain = get_domain(layer.service.url)
    if hasattr(layer, 'layerwm'):
        category = layer.layerwm.category
        username = layer.layerwm.username
    abstract = layer.abstract
    if abstract:
        abstract = strip_tags(layer.abstract)
    else:
        abstract = ''
    if layer.type == "WM":
        originator = username
    else:
        originator = domain

    layer_dict = {
                    'id': layer.id,
                    'type': 'Layer',
                    'layer_id': layer.id,
                    'name': layer.name,
                    'title': layer.title,
                    'layer_originator': originator,
                    'service_id': layer.service.id,
                    'service_type': layer.service.type,
                    'layer_category': category,
                    'layer_username': username,
                    'url': layer.url,
                    'reliability': layer.reliability,
                    'recent_reliability': layer.recent_reliability,
                    'last_status': layer.last_status,
                    'is_public': layer.is_public,
                    'availability': 'Online',
                    'location': '{"layerInfoPage": "' + layer.get_absolute_url + '"}',
                    'abstract': abstract,
                    'domain_name': layer.service.get_domain
                    }

    solr_date, date_type = get_date(layer)
    if solr_date is not None:
        layer_dict['layer_date'] = solr_date
        layer_dict['layer_datetype'] = date_type
    if bbox is not None:
        layer_dict['min_x'] = minX
        layer_dict['min_y'] = minY
        layer_dict['max_x'] = maxX
        layer_dict['max_y'] = maxY
        layer_dict['area'] = area
        layer_dict['bbox'] = wkt
        layer_dict['centroid_x'] = rectangle.centroid.x
        layer_dict['centroid_y'] = rectangle.centroid.y
        srs_list = [srs.encode('utf-8') for srs in layer.service.srs.values_list('code', flat=True)]
        layer_dict['srs'] = srs_list
    if layer.get_tile_url():
        layer_dict['tile_url'] = layer.get_tile_url()

    return layer_dict


def detect_metadata_url_scheme(url):
    """detect whether a url is a Service type that HHypermap supports"""

    scheme = None
    url_lower = url.lower()

    if any(x in url_lower for x in ['wms', 'service=wms']):
        scheme = 'OGC:WMS'
    if any(x in url_lower for x in ['wmts', 'service=wmts']):
        scheme = 'OGC:WMTS'
    elif all(x in url for x in ['/MapServer', 'f=json']):
        scheme = 'ESRI:ArcGIS:MapServer'
    elif all(x in url for x in ['/ImageServer', 'f=json']):
        scheme = 'ESRI:ArcGIS:ImageServer'

    return scheme



from django.contrib import admin

# Register your models here.
from models import Dynasty


class DynastyAdmin(admin.ModelAdmin):
    model = Dynasty
    list_display = ('date_range', 'name')

admin.site.register(Dynasty, DynastyAdmin)
